@@ -19,47 +19,49 @@
 #include <string>
 #include <unordered_map>
 
-#include <qcstring.h>
-
+#include "qcstring.h"
 #include "linkedmap.h"
+#include "construct.h"
 
 class Definition;
 
-//! enum representing the various types of sections and entities that can be referred to.
-enum class SectionType
+class SectionType
 {
-  Page          = 0,
-  Section       = 1,
-  Subsection    = 2,
-  Subsubsection = 3,
-  Paragraph     = 4,
-  Anchor        = 5,
-  Table         = 6
-};
+  public:
+    static constexpr int Page            = 0;
+    static constexpr int MinLevel        = 1;
+    static constexpr int Section         = 1;
+    static constexpr int Subsection      = 2;
+    static constexpr int Subsubsection   = 3;
+    static constexpr int Paragraph       = 4;
+    static constexpr int Subparagraph    = 5;
+    static constexpr int Subsubparagraph = 6;
+    static constexpr int MaxLevel        = 6;
+    static constexpr int Anchor          = 7;
+    static constexpr int Table           = 8;
 
-//! return true if type is a section, and false if it is a page, anchor or table.
-inline constexpr bool isSection(SectionType type)
-{
-  return (type==SectionType::Section       ||
-          type==SectionType::Subsection    ||
-          type==SectionType::Subsubsection ||
-          type==SectionType::Paragraph);
-}
+    constexpr SectionType() : m_level(0) {}
+    constexpr SectionType(int lvl) : m_level(lvl) {}
+    constexpr int level() const { return m_level; }
+    constexpr bool isSection() const
+    {
+      return m_level>=SectionType::MinLevel && m_level<=SectionType::MaxLevel;
+    }
+
+  private:
+    int m_level;
+};
 
 //! class that provide information about a section.
 class SectionInfo
 {
   public:
-    SectionInfo(const char *label, const char *fileName, int lineNr,
-                const char *title, SectionType type, int level,const char *ref) :
+    SectionInfo(const QCString &label, const QCString &fileName, int lineNr,
+                const QCString &title, SectionType type, int level,const QCString &ref) :
         m_label(label), m_title(title), m_type(type), m_ref(ref),
         m_lineNr(lineNr), m_fileName(fileName), m_level(level)
     {
-      //printf("SectionInfo(%p)\n",this);
-    }
-    ~SectionInfo()
-    {
-      //printf("~SectionInfo(%p)\n",this);
+      //printf("SectionInfo(%p) fileName=%s\n",(void*)this,qPrint(fileName));
     }
 
     // getters
@@ -74,10 +76,14 @@ class SectionInfo
     Definition *definition() const { return m_definition; }
 
     // setters
-    void setFileName(const char *fn)  { m_fileName   = fn; }
-    void setType(SectionType t)       { m_type       = t;  }
-    void setGenerated(bool b)         { m_generated  = b;  }
-    void setDefinition(Definition *d) { m_definition = d;  }
+    void setFileName(const QCString &fn) { m_fileName   = fn; }
+    void setType(SectionType t)          { m_type       = t;  }
+    void setGenerated(bool b)            { m_generated  = b;  }
+    void setDefinition(Definition *d)    { m_definition = d;  }
+    void setTitle(const QCString &t)     { m_title      = t;  }
+    void setLevel(int l)                 { m_level      = l;  }
+    void setReference(const QCString &r) { m_ref        = r;  }
+    void setLineNr(int l)                { m_lineNr     = l;  }
 
   private:
     QCString    m_label;
@@ -88,7 +94,7 @@ class SectionInfo
     QCString    m_fileName;
     bool        m_generated = false;
     int         m_level;
-    Definition *m_definition = 0;
+    Definition *m_definition = nullptr;
 };
 
 //! class that represents a list of constant references to sections.
@@ -100,9 +106,9 @@ class SectionRefs
 
     //! Returns a constant pointer to the section info given a section label or nullptr
     //! if no section with the given label can be found.
-    const SectionInfo *find(const char *label) const
+    const SectionInfo *find(const QCString &label) const
     {
-      auto it = m_lookup.find(label);
+      auto it = m_lookup.find(label.str());
       return it!=m_lookup.end() ? it->second : nullptr;
     }
 
@@ -116,7 +122,7 @@ class SectionRefs
     const_iterator begin() const { return m_entries.cbegin(); }
     const_iterator end()   const { return m_entries.cend(); }
     bool empty() const { return m_entries.empty(); }
-    int size() const   { return (int)m_entries.size(); }
+    size_t size() const { return m_entries.size(); }
 
   private:
     SectionInfoVec m_entries;
@@ -131,15 +137,38 @@ class SectionManager : public LinkedMap<SectionInfo>
     //! Returns a non-owning pointer to the newly added section.
     SectionInfo *add(const SectionInfo &si)
     {
-      return LinkedMap<SectionInfo>::add(si.label(),si.fileName(),si.lineNr(),si.title(),si.type(),si.level(),si.ref());
+      return LinkedMap<SectionInfo>::add(si.label(),si.fileName(),
+                      si.lineNr(),si.title(),si.type(),si.level(),si.ref());
     }
 
     //! Add a new section
     //! Return a non-owning pointer to the newly added section
-    SectionInfo *add(const char *label, const char *fileName, int lineNr,
-                     const char *title, SectionType type, int level,const char *ref=0)
+    SectionInfo *add(const QCString &label, const QCString &fileName, int lineNr,
+                     const QCString &title, SectionType type, int level,const QCString &ref=QCString())
     {
-      return LinkedMap<SectionInfo>::add(label,fileName,lineNr,title,type,level,ref);
+      return LinkedMap<SectionInfo>::add(label.data(),fileName,lineNr,title,type,level,ref);
+    }
+
+    //! Replace an existing section with a new one
+    //! Return a non-owning pointer to the newly added section
+    SectionInfo *replace(const QCString &label, const QCString &fileName, int lineNr,
+                         const QCString &title, SectionType type, int level,const QCString &ref=QCString())
+    {
+      SectionInfo *si = LinkedMap<SectionInfo>::find(label.data());
+      if (si)
+      {
+        si->setFileName(fileName);
+        si->setLineNr(lineNr);
+        si->setTitle(title);
+        si->setType(type);
+        si->setLevel(level);
+        si->setReference(ref);
+        return si;
+      }
+      else
+      {
+        return LinkedMap<SectionInfo>::add(label.data(),fileName,lineNr,title,type,level,ref);
+      }
     }
 
     //! returns a reference to the singleton
@@ -150,9 +179,9 @@ class SectionManager : public LinkedMap<SectionInfo>
     }
 
   private:
-    SectionManager() {}
-    SectionManager(const SectionManager &other) = delete;
-    SectionManager &operator=(const SectionManager &other) = delete;
+    SectionManager() = default;
+   ~SectionManager() = default;
+    NON_COPYABLE(SectionManager)
 };
 
 
