@@ -20,18 +20,21 @@
 
 #include "classdef.h"
 #include "config.h"
+#include "datetime.h"
+#include "index.h"
+#include "construct.h"
 
 /** Abstract base class for all translatable text fragments. */
 class Translator
 {
   public:
+    ABSTRACT_BASE_CLASS(Translator)
 
     /*! This method is used to provide warning message that is displayed
      *  if the user chooses a language whose translation is not up to date.
      *  It is implemented by the adapter classes.
      */
     virtual QCString updateNeededMessage() { return QCString(); }
-    virtual ~Translator() {}
 
     // Please, have a look at comments inside the translator_en.h file
     // to learn the meaning of the following methods.  The translator_en.h
@@ -69,21 +72,93 @@ class Translator
      */
     virtual QCString latexCommandName()
     {
-      QCString latex_command = Config_getString(LATEX_CMD_NAME);
-      if (latex_command.isEmpty()) latex_command = "latex";
-      if (Config_getBool(USE_PDFLATEX))
-      {
-        if (latex_command == "latex") latex_command = "pdflatex";
-      }
-      return latex_command;
+      return p_latexCommandName("pdflatex");
     }
     virtual QCString trISOLang() = 0;
 
+    /** language codes for Html help
+
+       <pre>
+       0x402 Bulgarian
+       0x405 Czech
+       0x406 Danish
+       0x413 Dutch
+       0xC09 English (Australia)
+       0x809 English (Britain)
+       0x1009 English (Canada)
+       0x1809 English (Ireland)
+       0x1409 English (New Zealand)
+       0x1C09 English (South Africa)
+       0x409 English (United States)
+       0x40B Finnish
+       0x40C French
+       0x407 German
+       0x408 Greece
+       0x439 Hindi
+       0x40E Hungarian
+       0x410 Italian
+       0x814 Norwegian
+       0x415 Polish
+       0x816 Portuguese(Portugal)
+       0x416 Portuguese(Brazil)
+       0x419 Russian
+       0x80A Spanish(Mexico)
+       0xC0A Spanish(Modern Sort)
+       0x40A Spanish(Traditional Sort)
+       0x41D Swedish
+       0x41F Turkey
+       0x411 Japanese
+       0x412 Korean
+       0x804 Chinese (PRC)
+       0x404 Chinese (Taiwan)
+       </pre>
+
+       New LCIDs:
+       <pre>
+       0x421 Indonesian
+       0x41A Croatian
+       0x418 Romanian
+       0x424 Slovenian
+       0x41B Slovak
+       0x422 Ukrainian
+       0x81A Serbian (Serbia, Latin)
+       0x403 Catalan
+       0x426 Latvian
+       0x427 Lithuanian
+       0x436 Afrikaans
+       0x42A Vietnamese
+       0x429 Persian (Iran)
+       0xC01 Arabic (Egypt) - I don't know which version of arabic is used inside translator_ar.h ,
+             so I have chosen Egypt at random
+       </pre>
+
+      Code for Esperanto should be as shown below but the htmlhelp compiler 1.3 does not support this
+      (and no newer version is available).
+      <pre>
+      0x48f Esperanto
+      </pre>
+      So do a fallback to the default language
+      <pre>
+      0x409 English (United States)
+      </pre>
+
+      <pre>
+      0xC1A Serbian (Serbia, Cyrillic)
+      </pre>
+    */
+    virtual QCString getLanguageString() = 0;
+
+    /**
+     * add punctuation at the end of a brief description when needed and supported by the language
+    */
+    virtual bool needsPunctuation() { return true; }
+
     // --- Language translation methods -------------------
 
     virtual QCString trRelatedFunctions() = 0;
     virtual QCString trRelatedSubscript() = 0;
     virtual QCString trDetailedDescription() = 0;
+    virtual QCString trDetails() = 0;
     virtual QCString trMemberTypedefDocumentation() = 0;
     virtual QCString trMemberEnumerationDocumentation() = 0;
     virtual QCString trMemberFunctionDocumentation() = 0;
@@ -104,7 +179,6 @@ class Translator
     virtual QCString trClassHierarchy() = 0;
     virtual QCString trCompoundList() = 0;
     virtual QCString trFileList() = 0;
-    //virtual QCString trHeaderFiles() = 0;
     virtual QCString trCompoundMembers() = 0;
     virtual QCString trFileMembers() = 0;
     virtual QCString trRelatedPages() = 0;
@@ -115,15 +189,13 @@ class Translator
     virtual QCString trCompoundListDescription() = 0;
     virtual QCString trCompoundMembersDescription(bool extractAll) = 0;
     virtual QCString trFileMembersDescription(bool extractAll) = 0;
-    //virtual QCString trHeaderFilesDescription() = 0;
     virtual QCString trExamplesDescription() = 0;
     virtual QCString trRelatedPagesDescription() = 0;
     virtual QCString trModulesDescription() = 0;
-    //virtual QCString trNoDescriptionAvailable() = 0;
 
     // index titles (the project name is prepended for these)
 
-    virtual QCString trDocumentation() = 0;
+    virtual QCString trDocumentation(const QCString &projName) = 0;
     virtual QCString trModuleIndex() = 0;
     virtual QCString trHierarchicalIndex() = 0;
     virtual QCString trCompoundIndex() = 0;
@@ -131,30 +203,22 @@ class Translator
     virtual QCString trModuleDocumentation() = 0;
     virtual QCString trClassDocumentation() = 0;
     virtual QCString trFileDocumentation() = 0;
-    virtual QCString trExampleDocumentation() = 0;
-    virtual QCString trPageDocumentation() = 0;
     virtual QCString trReferenceManual() = 0;
     virtual QCString trDefines() = 0;
-    //virtual QCString trFuncProtos() = 0;
     virtual QCString trTypedefs() = 0;
     virtual QCString trEnumerations() = 0;
     virtual QCString trFunctions() = 0;
     virtual QCString trVariables() = 0;
     virtual QCString trEnumerationValues() = 0;
     virtual QCString trDefineDocumentation() = 0;
-    //virtual QCString trFunctionPrototypeDocumentation() = 0;
     virtual QCString trTypedefDocumentation() = 0;
     virtual QCString trEnumerationTypeDocumentation() = 0;
     virtual QCString trFunctionDocumentation() = 0;
     virtual QCString trVariableDocumentation() = 0;
     virtual QCString trCompounds() = 0;
     virtual QCString trGeneratedAt(const QCString &date,const QCString &projName) = 0;
-    //virtual QCString trWrittenBy() = 0;
     virtual QCString trClassDiagram(const QCString &clName) = 0;
-    virtual QCString trForInternalUseOnly() = 0;
-    //virtual QCString trReimplementedForInternalReasons() = 0;
     virtual QCString trWarning() = 0;
-    //virtual QCString trBugsAndLimitations() = 0;
     virtual QCString trVersion() = 0;
     virtual QCString trDate() = 0;
     virtual QCString trReturns() = 0;
@@ -220,7 +284,6 @@ class Translator
 
     virtual QCString trGeneratedFromFiles(ClassDef::CompoundType compType,
                                           bool single) = 0;
-    //virtual QCString trAlphabeticalList() = 0;
 
 //////////////////////////////////////////////////////////////////////////
 // new since 0.49-990901
@@ -234,7 +297,6 @@ class Translator
 // new since 0.49-991003
 //////////////////////////////////////////////////////////////////////////
 
-    //virtual QCString trSources() = 0;
     virtual QCString trDefinedAtLineInSourceFile() = 0;
     virtual QCString trDefinedInSourceFile() = 0;
 
@@ -311,12 +373,6 @@ class Translator
     virtual QCString trTest() = 0;
     virtual QCString trTestList() = 0;
 
-//////////////////////////////////////////////////////////////////////////
-// new since 1.2.1
-//////////////////////////////////////////////////////////////////////////
-
-    //virtual QCString trDCOPMethods() = 0;
-
 //////////////////////////////////////////////////////////////////////////
 // new since 1.2.2
 //////////////////////////////////////////////////////////////////////////
@@ -328,13 +384,10 @@ class Translator
 // new since 1.2.4
 //////////////////////////////////////////////////////////////////////////
 
-    //virtual QCString trInterfaces() = 0;
     virtual QCString trClasses() = 0;
     virtual QCString trPackage(const QCString &name) = 0;
-    virtual QCString trPackageList() = 0;
     virtual QCString trPackageListDescription() = 0;
     virtual QCString trPackages() = 0;
-    //virtual QCString trPackageDocumentation() = 0;
     virtual QCString trDefineValue() = 0;
 
 //////////////////////////////////////////////////////////////////////////
@@ -415,7 +468,6 @@ class Translator
     virtual QCString trGroup(bool first_capital, bool singular) = 0;
     virtual QCString trPage(bool first_capital, bool singular) = 0;
     virtual QCString trMember(bool first_capital, bool singular) = 0;
-    //virtual QCString trField(bool first_capital, bool singular) = 0;
     virtual QCString trGlobal(bool first_capital, bool singular) = 0;
 
 //////////////////////////////////////////////////////////////////////////
@@ -461,8 +513,9 @@ class Translator
 //////////////////////////////////////////////////////////////////////////
 
     virtual QCString trPackageTypes() = 0;
+    virtual QCString trPackageFunctions() = 0;
     virtual QCString trPackageMembers() = 0;
-    virtual QCString trStaticPackageMembers() = 0;
+    virtual QCString trStaticPackageFunctions() = 0;
     virtual QCString trPackageAttribs() = 0;
     virtual QCString trStaticPackageAttribs() = 0;
 
@@ -477,7 +530,6 @@ class Translator
 // new since 1.3.3
 //////////////////////////////////////////////////////////////////////////
 
-    //virtual QCString trSearchForIndex() = 0;
     virtual QCString trSearchResultsTitle() = 0;
     virtual QCString trSearchResults(int numDocuments) = 0;
     virtual QCString trSearchMatches() = 0;
@@ -486,7 +538,7 @@ class Translator
 // new since 1.3.8
 //////////////////////////////////////////////////////////////////////////
 
-    virtual QCString trSourceFile(QCString& filename) = 0;
+    virtual QCString trSourceFile(const QCString& filename) = 0;
 
 //////////////////////////////////////////////////////////////////////////
 // new since 1.3.9
@@ -495,7 +547,6 @@ class Translator
     virtual QCString trDirIndex() = 0;
     virtual QCString trDirDocumentation() = 0;
     virtual QCString trDirectories() = 0;
-    virtual QCString trDirDescription() = 0;
     virtual QCString trDirReference(const QCString &dirName) = 0;
     virtual QCString trDir(bool first_capital, bool singular) = 0;
 
@@ -556,12 +607,14 @@ class Translator
 // new since 1.6.3
 //////////////////////////////////////////////////////////////////////////
 
-    //virtual QCString trDirDependency(const QCString &name) = 0;
     virtual QCString trFileIn(const QCString &name) = 0;
     virtual QCString trIncludesFileIn(const QCString &name) = 0;
     virtual QCString trDateTime(int year,int month,int day,int dayOfWeek,
                                 int hour,int minutes,int seconds,
-                                bool includeTime) = 0;
+                                DateTimeType includeTime) = 0;
+    virtual QCString trDayOfWeek(int dayOfWeek, bool first_capital, bool full) = 0;
+    virtual QCString trMonth(int month, bool first_capital, bool full) = 0;
+    virtual QCString trDayPeriod(bool period) = 0;
 
 //////////////////////////////////////////////////////////////////////////
 // new since 1.7.5
@@ -618,7 +671,7 @@ class Translator
     virtual QCString trDesignUnitIndex() = 0;
     virtual QCString trDesignUnits() = 0;
     virtual QCString trFunctionAndProc() = 0;
-    virtual QCString trVhdlType(uint64 type,bool single) = 0;
+    virtual QCString trVhdlType(VhdlSpecifier type,bool single) = 0;
     virtual QCString trCustomReference(const QCString &name) = 0;
 
     virtual QCString trConstants() = 0;
@@ -668,6 +721,83 @@ class Translator
     virtual QCString trConceptDocumentation() = 0;
     virtual QCString trConceptListDescription(bool extractAll) = 0;
     virtual QCString trConceptDefinition() = 0;
+
+//////////////////////////////////////////////////////////////////////////
+// new since 1.9.4
+//////////////////////////////////////////////////////////////////////////
+    virtual QCString trPackageList() = 0;
+
+//////////////////////////////////////////////////////////////////////////
+// new since 1.9.6
+//////////////////////////////////////////////////////////////////////////
+    virtual QCString trFlowchart() = 0;
+
+//////////////////////////////////////////////////////////////////////////
+// new since 1.9.7
+//////////////////////////////////////////////////////////////////////////
+    virtual QCString trRelatedSymbols() = 0;
+    virtual QCString trRelatedSymbolsSubscript() = 0;
+    virtual QCString trRelatedSymbolDocumentation() = 0;
+
+    virtual QCString trCompoundType(ClassDef::CompoundType compType, SrcLangExt lang) = 0;
+
+    virtual QCString trFileMembersDescriptionTotal(FileMemberHighlight::Enum hl) = 0;
+    virtual QCString trCompoundMembersDescriptionTotal(ClassMemberHighlight::Enum hl) = 0;
+    virtual QCString trNamespaceMembersDescriptionTotal(NamespaceMemberHighlight::Enum hl) = 0;
+    virtual QCString trDefinition() = 0;
+    virtual QCString trDeclaration() = 0;
+//////////////////////////////////////////////////////////////////////////
+// new since 1.9.8
+//////////////////////////////////////////////////////////////////////////
+    virtual QCString trTopics() = 0;
+    virtual QCString trTopicDocumentation() = 0;
+    virtual QCString trTopicList() = 0;
+    virtual QCString trTopicIndex() = 0;
+    virtual QCString trTopicListDescription() = 0;
+    virtual QCString trModuleMembersDescriptionTotal(ModuleMemberHighlight::Enum hl) = 0;
+    virtual QCString trExportedModules() = 0;
+//////////////////////////////////////////////////////////////////////////
+// new since 1.10.0
+//////////////////////////////////////////////////////////////////////////
+    virtual QCString trCopyToClipboard() = 0;
+//////////////////////////////////////////////////////////////////////////
+// new since 1.11.0
+//////////////////////////////////////////////////////////////////////////
+    virtual QCString trImportant() = 0;
+
+  protected:
+    QCString p_latexCommandName(const QCString &latexCmd)
+    {
+      QCString latex_command = Config_getString(LATEX_CMD_NAME);
+      if (latex_command.isEmpty()) latex_command = "latex";
+      if (Config_getBool(USE_PDFLATEX))
+      {
+        if (latex_command == "latex") latex_command = latexCmd;
+      }
+      return latex_command;
+    }
+    /*! For easy flexible-noun implementation.
+     *  \internal
+     */
+    QCString createNoun(bool first_capital, bool singular,
+                        const QCString &base,
+                        const QCString &plurSuffix, const QCString &singSuffix = "" )
+    {
+      QCString result;
+      if (first_capital)
+      {
+        std::string res = getUTF8CharAt(base.str(),0);
+        res = convertUTF8ToUpper(res);
+        result = res.c_str();
+        result += base.mid(res.length());
+      }
+      else
+      {
+        result = base;
+      }
+      result += (singular ? singSuffix : plurSuffix);
+      return result;
+    }
 };
 
 #endif
